package de.UllisRoboterSeite.UrsAI2Battery;

// Autor: http://UllisRoboterSeite.de
// Doku:  http://UllisRoboterSeite.de/android-AI2-Battery-de.html
// Created: 2020-01-25
//
// Version 1 (2020-01-25)
// -------------------------
// - Basis-Version
//
// Version 1.1 (2020-03-07)
// -------------------------
// - URL zur Hilfe war nicht korrekt

import java.util.*;

import com.google.appinventor.components.annotations.*;
import com.google.appinventor.components.common.*;
import com.google.appinventor.components.runtime.*;
import android.os.BatteryManager;
import android.content.Context;

import android.content.Intent;
import android.content.IntentFilter;
import android.app.Activity;
import android.content.BroadcastReceiver;
import android.os.Handler;
import java.text.DecimalFormat;

@DesignerComponent(version = 1, //
        versionName = "1.1", //
        dateBuilt = "2020-01-25", //
        description = "AI2 extension block for battery informations.", //
        category = com.google.appinventor.components.common.ComponentCategory.EXTENSION, //
        nonVisible = true, //
        helpUrl = "http://UllisRoboterSeite.de/android-AI2-Battery-de.html", //
        iconName = "aiwebres/battery.png")
@UsesPermissions(permissionNames = "android.permission.BATTERY_STATS")
@SimpleObject(external = true)
public class UrsAI2Battery extends AndroidNonvisibleComponent implements Component {
    Context thisContext = null;

    final Handler handler = new Handler();
    final UrsAI2Battery thisInstance = this;
    IntentFilter intentfilter;

    public UrsAI2Battery(ComponentContainer container) {
        super(container.$form());
        thisContext = container.$context();
        intentfilter = new IntentFilter(Intent.ACTION_BATTERY_CHANGED);
        Activity ac = (Activity) thisContext;
        ac.registerReceiver(broadcastreceiver, intentfilter);
    } // ctor

    @SimpleProperty(description = "Returns the remaining battery energy in mWh.\n-1 if not supported.")
    public int RemainingEnergy() {
        BatteryManager mBatteryManager = (BatteryManager) thisContext.getSystemService(Context.BATTERY_SERVICE);
        long energy = mBatteryManager.getLongProperty(BatteryManager.BATTERY_PROPERTY_ENERGY_COUNTER);
        if (energy == Long.MIN_VALUE)
            return -1;
        return (int) (energy / 1000000);
    }

    @SimpleProperty(description = "Return the remaining battery capacity as an integer percentage.\n-1 if not supported.")
    public int RemainingCapacity() {
        BatteryManager mBatteryManager = (BatteryManager) thisContext.getSystemService(Context.BATTERY_SERVICE);
        long pct = mBatteryManager.getLongProperty(BatteryManager.BATTERY_PROPERTY_CAPACITY);
        if (pct == Long.MIN_VALUE)
            return -1;
        return (int) pct;
    }

    @SimpleProperty(description = "Return the Remaining battery capacity in milliampere-hours.\n-1 if not supported.")
    public int RemainingCharge() {
        BatteryManager mBatteryManager = (BatteryManager) thisContext.getSystemService(Context.BATTERY_SERVICE);
        long charge = mBatteryManager.getLongProperty(BatteryManager.BATTERY_PROPERTY_CHARGE_COUNTER);
        if (charge == Long.MIN_VALUE)
            return -1;
        return (int) (charge / 1000);
    }

    @SimpleProperty(description = "Returns true if the battery is currently considered to be charging.")
    public boolean IsCharging() {
        BatteryManager mBatteryManager = (BatteryManager) thisContext.getSystemService(Context.BATTERY_SERVICE);
        return mBatteryManager.isCharging();
    }

    @SimpleProperty(description = "Returns the instantaneous battery current in milliamperes.\n-99999 if not supported.\n"
            + "Positive values indicate battery is discharging, negative values indicate charging.")
    public int CurrentNow() {
        BatteryManager mBatteryManager = (BatteryManager) thisContext.getSystemService(Context.BATTERY_SERVICE);
        long current = mBatteryManager.getLongProperty(BatteryManager.BATTERY_PROPERTY_CURRENT_NOW);
        if (current == Long.MIN_VALUE)
            return -99999;
        return (int) (current / 1000);
    }

    @SimpleProperty(description = "Returns the average battery current in milliamperes.\n-99999 if not supported.\n"
            + "Positive values indicate battery is discharging, negative values indicate charging.\n"
            + "The time period over which the average is computed may depend on the fuel gauge hardware and its configuration.")
    public int CurrentAverage() {
        BatteryManager mBatteryManager = (BatteryManager) thisContext.getSystemService(Context.BATTERY_SERVICE);
        long current = mBatteryManager.getLongProperty(BatteryManager.BATTERY_PROPERTY_CURRENT_AVERAGE);
        if (current == Long.MIN_VALUE)
            return -99999;
        return (int) (current / 1000);
    }

    int mVoltage = -1;

    @SimpleProperty(description = "Returns the current voltage in mV.\n-1 if not supported.\n")
    public int Voltage() {
        return mVoltage;
    }

    private BroadcastReceiver broadcastreceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            int voltageValue;
            voltageValue = intent.getIntExtra(BatteryManager.EXTRA_VOLTAGE, 0);
            mVoltage = (int) voltageValue;
        } // onReceive
    };
}